#ifndef _AVSYNCHRONIZER_FISHEYE_H
#define _AVSYNCHRONIZER_FISHEYE_H

#include "typedef.h"
#include "AVSynchronizer.h"

// ENUMERRATION and STRUCTURE
// ----------------------------------------------------------------------------------------------------
typedef enum _FisheyePrensentMode
{
	FEPRESENTMODE_SINGLE_SOURCE_STRETCH          = 0,   // Single view: stretched original source
	FEPRESENTMODE_SINGLE_SOURCE_CONSTRAINPROP    = 1,   // Single view: original source with unchanged aspect ratio
	FEPRESENTMODE_SINGLE_PERSPECT_VIEW           = 2,   // Single view: perspective
	FEPRESENTMODE_SINGLE_PANORAMA_VIEW           = 3,   // Single view: panorama
	FEPRESENTMODE_DOUBLE_SOURCE_PERSPECT         = 4,   // Double views: one original source with unchanged aspect ratio, another is perspective
	FEPRESENTMODE_DOUBLE                         = 5,   // Double views: two panorama
	FEPRESENTMODE_TRIPLE                         = 6,   // Triple views: one panorama and two perspective
	FEPRESENTMODE_QUAD_WITH_SOURCE_CONSTRAINPROP = 7,   // Quadruple views: one original source with unchanged aspect ratio and three perspective
	FEPRESENTMODE_QUAD_PERSPECT_VIEW             = 8,   // Quadruple views: four perspective with individual control
	FEPRESENTMODE_QUAD_PERSPECT_VIEW_INTERFERED  = 9,   // Quadruple views: four perspective with interfered control
	FEPRESENTMODE_QUAD_WITH_PANORAMA_VIEW        = 10,  // Quadruple views: one panorama view and three perspective
	FEPRESENTMODE_NINE                           = 11   // Nine views: one original source with unchanged aspect ratio and eight perspective
}EFISHEYEPRESENTMODE;

typedef enum _FisheyePTZRControlFlags
{
	FEPTZRCONTROL_PTR       = 1,   // Pan, tilt and rotate
	FEPTZRCONTROL_Z         = 2,   // Zoom 
	FEPTZRCONTROL_RESET     = 4,   // Reset
	FEPTZRCONTROL_CLICK     = 8,   // Click-on-image control
	FEPTZRCONTROL_ANCHOR    = 16,  // Set anchor (focus) to quadrant
	FEPTZRCONTROL_AUTOSPEED = 32   // Set the speed of auto panoramic view (degree/unit time)
}EFISHEYEPTZRCONTROLFLAGS;

typedef struct _FisheyePTZRControlCommands
{	
	DWORD dwCtrlFlags;      // Control flags, which is a combination of one or more EFISHEYEPTZRCONTROLFLAGS constants

	BOOL bClick;            // Click-on-image control, the EndPoint in PTRParams is omitted when user enable the click-on-image control

	LONG autoSpeed;         // auto panoramic view speed (degree)
	
	POINTS AnchorPoint;     // use anchor point to locate the focus quadrant    
	
	struct 
	{
		POINTS BeginPoint;  // Begin point, which is often set when press the left mouse button in the client area of viewer's window
		POINTS EndPoint;    // End point, which is often set when the cursor moves
		BOOL   bTrack;      // Indicate if the cursor starts moving or stop, which is often set to TRUE when press the left mouse button and FALSE when the left mouse button is up
	}PTRParams;

	struct
	{		
		POINTS FocusPoint;  // Focus point, which is set to the cursor's position in the client area of viewer's window 
		SHORT  DeltaSteps;  // Steps of zoom-in or zoom-out operation 
	}ZParams;

	struct
	{
		POINTS FocusPoint;  // Focus point, which is set to the cursor's position in the client area of viewer's window 
		BOOL   bResetP;     // Determine whether reset parameters of pan
		BOOL   bResetT;     // Determine whether reset parameters of tilt
		BOOL   bResetZ;     // Determine whether reset parameters of zoom
		BOOL   bResetR;     // Determine whether reset parameters of rotate
	}ResetParams;

}TFISHEYEPTZRCONTROL_COMMANDS;

typedef enum _FisheyeViewerAngleFlags
{
	FEVIEWERANGLE_180 = 0,  // 180 degree viewer angle (wall)
	FEVIEWERANGLE_360 = 1,  // 360 degree viewer angle (ceiling)
	FEVIEWERANGLE_360I = 2  // 360 degree viewer angle (floor)
}EFISHEYEVIEWERANGLEFLAGS;

typedef enum _FisheyeUpdateViewerFlags
{
	FEUPDATEVIEWER_PRESENTMODE     = 1,  // Present mode
	FEUPDATEVIEWER_VIEWERANGLE     = 2,  // Viewer angle
	FEUPDATEVIEWER_CONTROLCOMMANDS = 4,  // Control commands
	FEUPDATEVIEWER_SHOWFOCUSREGION = 8,  // Show focus region
}EFISHEYEUPDATEVIEWERFLAGS;

typedef struct _FisheyeViewer
{
	DWORD                        dwUpdateFlags;  // Update flags, which is a combination of one or more EFISHEYEUPDATEVIEWERFLAGS constants
	HWND                         hViewerWindow;  // Viewer's window
	EFISHEYEPRESENTMODE          dwPresentMode;  // Viewer's present mode, which is one of EFISHEYEPRESENTMODE constants
	EFISHEYEVIEWERANGLEFLAGS     dwViewerAngle;  // Viewer angle, which is one of EFISHEYEVIEWERANGLEFLAGS constants
	TFISHEYEPTZRCONTROL_COMMANDS tCtrlCommands;  // Control commands, describing the PTZR parameters
	DWORD                        FocusRgnColor;  // Focus region color, using RGB macro to select a color
}TFISHEYEVIEWER;

typedef struct _FisheyeSnapshot
{
	TSNAPSHOT tSnapshot;      // Snapshot information
	HWND      hViewerWindow;  // Viewer's window
}TFISHEYESNAPSHOT;

typedef struct _FisheyeCurrentDisplay
{
	TCURRENTDISPLAYINFO tCurrentDisplay;  // Current display information
	HWND                hViewerWindow;    // Viewer's window
}TFISHEYECURRENTDISPLAY;

typedef struct _FisheyeControlParameters
{
	LONG ThetaX;       // Rotate angle around the x-axis (Tilt)
	LONG ThetaY;       // Rotate angle around the y-axis (Pan)
	LONG ThetaZ;       // Rotate angle around the z-axis (Rotate)
	LONG ZoomFactor;   // Zoom factor along the x-axis and y-axis
	LONG ThetaScroll;  // Scroll angle (only valid in 360 degree panorama projection)
}TFISHEYECONTROLPARAMS;

typedef struct _FisheyePresentParameters
{
	HWND                     hViewerWindow;     // [in]  Viewer's window
	BOOL                     bResetCtrlParams;  // [in]  Reset the control parameters in focus quadrant
	EFISHEYEPRESENTMODE      PresentMode;       // [out] Viewer's present mode, which is one of EFISHEYEPRESENTMODE constants
	EFISHEYEVIEWERANGLEFLAGS ViewerAngle;       // [out] Viewer angle, which is one of EFISHEYEVIEWERANGLEFLAGS constants
	TFISHEYECONTROLPARAMS    *pCtrlParams;      // [out] A pointer to an array of TFISHEYECONTROLPARAMS structures
	DWORD                    CtrlParamsCount;   // [out] The number of control parameters in the array
	DWORD                    FocusQuadrant;     // [out] The focus quadrant 
}TFISHEYEPRESENTPARAMS;

typedef enum _FisheyeUpdateRotateOptFlags
{
	FEUPDATEROTATE_POINTS = 1,  // Rotating point's array
	FEUPDATEROTATE_TARGET = 2,  // Target window
	FEUPDATEROTATE_SPEED  = 4   // Rotate speed
}EFISHEYEUPDATEROTATEOPTFLAGS;

typedef struct _FisheyeRotateOptions
{
	DWORD                 dwUpdateFlags;        // Update flags, which is a combination of one or more EFISHEYEUPDATEROTATEOPTFLAGS constants
	TFISHEYECONTROLPARAMS *pPresetPoints;       // [in] A pointer to an array of TFISHEYECONTROLPARAMS structures
	DWORD                 PresetPointsCount;    // [in] The number of preset points in the array
	HWND                  hRotateTargetWindow;  // [in] Target window
	DWORD                 RotateSpeed;          // [in] The period (sec) of rotate 
}TFISHEYEROTATEOPTIONS;

typedef struct _FisheyePPTZParameters
{
	POINT tPosition;  // [out] The target point
	int nRadius;      // [out] The radius of target roi 
	SIZE tVideoSize;  // [out] The window's client size
	SIZE tResolution; // [out] The resolution of streaming
	int bStretch;     // [out] TRUE: use window's client size as coordinate system, FALSE: use resolution as coordinate system
}TFISHEYEPPTZPARAMS;
// ----------------------------------------------------------------------------------------------------

// API

#ifdef __cplusplus
extern "C" {
#endif
// ----------------------------------------------------------------------------------------------------
//< Check whether the machine supports fisheye renderer 
SCODE DLLAPI AvSynchronizer_FisheyeCheckSupport(BOOL *pbIsSupportFisheye);

//< Enable fisheye renderer
SCODE DLLAPI AvSynchronizer_FisheyeEnableRenderer(HANDLE hChannel);

//< Disable fisheye renderer (enable DirectDraw renderer implicitly)
SCODE DLLAPI AvSynchronizer_FisheyeDisableRenderer(HANDLE hChannel);

//< Register viewer
SCODE DLLAPI AvSynchronizer_FisheyeRegisterViewer(HANDLE hChannel, TFISHEYEVIEWER *ptFisheyeViewer);

//< Update viewer parameters
SCODE DLLAPI AvSynchronizer_FisheyeUpdateViewer(HANDLE hChannel, TFISHEYEVIEWER *ptFisheyeViewer);

//< Remove viewer
SCODE DLLAPI AvSynchronizer_FisheyeRemoveViewer(HANDLE hChannel, TFISHEYEVIEWER *ptFisheyeViewer);

//< Get the snapshot of specified viewer
SCODE DLLAPI AvSynchronizer_FisheyeGetCurrentViewerSnapShot(HANDLE hChannel, TFISHEYESNAPSHOT *ptFisheyeSnapshot, DWORD dwFlags);

//< Get the current display content of specified viewer
SCODE DLLAPI AvSynchronizer_FisheyeGetCurrentViewerDisplay(HANDLE hChannel, HDC hDC, TFISHEYECURRENTDISPLAY *ptFisheyeCurrentDisplay);

//< Get the present parameters of specified viewer
SCODE DLLAPI AvSynchronizer_FisheyeGetViewerPresentParameters(HANDLE hChannel, TFISHEYEPRESENTPARAMS *ptPresentParams);

//< Set the present parameters of specified viewer
SCODE DLLAPI AvSynchronizer_FisheyeSetViewerPresentParameters(HANDLE hChannel, TFISHEYEPRESENTPARAMS *ptPresentParams);

//< Update the rotate options
SCODE DLLAPI AvSynchronizer_FisheyeUpdateRotateOptions(HANDLE hChannel, TFISHEYEROTATEOPTIONS *ptRotateOptions);

//< Enable/Disable fisheye render to display the zoom ratio
SCODE DLLAPI AvSynchronizer_FisheyeSetZoomRatioDisplay(HANDLE hChannel, BOOL bDisplay);

//< Get the Panoramic PTZ URL control parameters
SCODE DLLAPI AvSynchronizer_FisheyeGetPPTZParameters(HANDLE hChannel, TFISHEYEVIEWER *ptFisheyeViewer, POINT *ptUsrPos, RECT *ptUsrRoi, TFISHEYEPPTZPARAMS *ptPPTZParams);

//< Get fisheye control parameters from motion polygon
SCODE DLLAPI AvSynchronizer_FisheyeGetControlParametersFromMotionPolygon(HANDLE hChannel, TFISHEYEVIEWER *ptFisheyeViewer, DWORD dwCount, const POINT *ptPoints, TFISHEYECONTROLPARAMS *ptCtrlParams);
// ----------------------------------------------------------------------------------------------------
#ifdef __cplusplus
}
#endif

#endif  // _AVSYNCHRONIZER_FISHEYE_H