#include <string.h>
#include "DataBroker.h"
#include "OSDefine.h"
#include "AVComposer.h"
#include "AVReader.h"

#define MAX_IP_LENGTH 256
#define MAX_USERNAME_LENGTH 256
#define MAX_PASSWORD_LENGTH 256
#define MAX_SERVERTYPE_LENGTH 16

#ifndef HTONL
#define HTONL(x) \
	(((DWORD)x) << 24) + \
	((((DWORD)x) & 0xff00) << 8) + \
	((((DWORD)x) >> 8) & 0xff00) + \
	(((DWORD)x) >> 24)
#endif

// This callback function is used to receive media packet, the calling convention must be stdcall
SCODE __stdcall DataBrokerAVCallBack(DWORD dwContext, TMediaDataPacketInfo *ptMediaDataPacket)
{
	// Cast dwContext to AVComposer handle
	AVComposer *pAVComposer((AVComposer*)dwContext);
	if (NULL == pAVComposer)
	{
		return S_FAIL;
	}

	SCODE scRet(S_OK);
	TMediaDataPacketInfoV3 *ptPacketV3((TMediaDataPacketInfoV3 *) ptMediaDataPacket);
	switch (ptPacketV3->tIfEx.tInfo.dwStreamType)
	{
	case mctJPEG:
	case mctH263:
	case mctMP4V:
	case mctH264:
	case mctHEVC:
		scRet = AVComposer_InputPacket(pAVComposer, ptPacketV3, eAVComposerTrack1, NULL);
		break;
	case mctG7221:
	case mctG729A:
	case mctAAC:
	case mctSAMR:
	case mctG711:
	case mctG711A:
	case mctG726:
		scRet = AVComposer_InputPacket(pAVComposer, ptPacketV3, eAVComposerTrack2, NULL);
		break;
	default:
		scRet = S_OK;
	}
	

	return scRet;
}

// This callback function is used to receive status info, the calling convention must be stdcall
SCODE __stdcall DataBrokerStatusCallBack(DWORD dwContext, TDataBrokerStatusType tStatusType, PVOID pvParam1, PVOID pvParam2)
{
	switch (tStatusType)
	{
		case eOnConnectionInfo:
		{
			printf("Get eOnConnectionInfo, start to receive packet\n");
		}
		break;

		case eOnStopped:
		{
			printf("Get eOnStopped, stop to receive packet\n");
		}
		break;

		default:
		{
		}	
		break;
	}

   return S_OK;
}

template<typename T>
void CreateConnectionOption(TDataBrokerConnectionOptions *ptOpt, T* pT)
{	
	//Set device IP
	printf("Please input device IP: ");
	fgets(ptOpt->pzIPAddr, MAX_IP_LENGTH, stdin);
	
	if (ptOpt->pzIPAddr[strlen(ptOpt->pzIPAddr) - 2] == '\r' && ptOpt->pzIPAddr[strlen(ptOpt->pzIPAddr) - 1] == '\n')
	{ 
	   ptOpt->pzIPAddr[strlen(ptOpt->pzIPAddr) - 2]= '\0';
	}
	else if (ptOpt->pzIPAddr[strlen(ptOpt->pzIPAddr) - 1] == '\n')
	{ 
	   ptOpt->pzIPAddr[strlen(ptOpt->pzIPAddr)-1]= '\0';
	}
        
	//Set device username
	printf("Please input device user name (press [ENTER] for empty): ");
	fgets(ptOpt->pzUID, MAX_USERNAME_LENGTH, stdin);
	
	if (ptOpt->pzUID[strlen(ptOpt->pzUID) - 2] == '\r' && ptOpt->pzUID[strlen(ptOpt->pzUID) - 1] == '\n')
	{ 
	   ptOpt->pzUID[strlen(ptOpt->pzUID) - 2]= '\0';
	}
	else if (ptOpt->pzUID[strlen(ptOpt->pzUID) - 1] == '\n')
	{ 
	   ptOpt->pzUID[strlen(ptOpt->pzUID)-1]= '\0';
	}

	//Set device password
	printf("Please input device password (press [ENTER] for empty): ");
	fgets(ptOpt->pzPWD, MAX_PASSWORD_LENGTH, stdin);
	
	if (ptOpt->pzPWD[strlen(ptOpt->pzPWD) - 2] == '\r' && ptOpt->pzPWD[strlen(ptOpt->pzPWD) - 1] == '\n')
	{ 
	   ptOpt->pzPWD[strlen(ptOpt->pzPWD) - 2]= '\0';
	}
	else if (ptOpt->pzPWD[strlen(ptOpt->pzPWD) - 1] == '\n')
	{ 
	   ptOpt->pzPWD[strlen(ptOpt->pzPWD)-1]= '\0';
	}


	//Set device HTTP port
	printf("Please input device HTTP port: ");
	_tscanf("%hu", &(ptOpt->wHttpPort));

	//Set connection protocol type to TCP, you may refer to TsdrProtocolType for other protocols.
	ptOpt->dwProtocolType = eptTCP;

	//Set connection media type to Video and Audio.
	ptOpt->dwMediaType =  (emtVideo | emtAudio);

	//Set device server type to auto, the DataBroker will auto detect the server type.
	strcpy(ptOpt->pzServerType, _T("auto"));

	//Pass context to DataBrokerStatusCallBack, the context is useful when we have to handle multi connections simultaneously.
	ptOpt->dwStatusContext = NULL;

	//Pass context to DataBrokerAVCallBack, the context is useful when we have to handle multi connections simultaneously.
	ptOpt->dwAVContext = (DWORD) pT;

	//Set status callback to connection option
	ptOpt->pfStatus = DataBrokerStatusCallBack;

	//Set Audio/Video callback to connection option
	ptOpt->pfAV = DataBrokerAVCallBack;

	//Set dwFlags to tell DataBroker what options we set.
	ptOpt->dwFlags = eConOptProtocolAndMediaType | eConOptHttpPort | 
		(eConOptStatusCallback | eConOptStatusContext) |
		(eConOptAVCallback | eConOptAVContext);
}

typedef struct
{
	int part;
#ifdef _UNICODE
	AVComposerW *pAVComposer;
#else
	AVComposer *pAVComposer;
#endif
} TFilenameInfo;

SCODE __stdcall GetNewFilename(HANDLE hContext, _TCHAR* pszNewFilename, TAVCOMPOSER_CLOSEFILEINFO* ptCurFileInfo)
{
	TFilenameInfo *pFilenameInfo = (TFilenameInfo*)hContext;
	_sntprintf(pszNewFilename, FILENAME_MAX, "Authenticate3GPFile_%d.3gp", ++pFilenameInfo->part);
	_tremove(pszNewFilename);
	// added watermark with different key for Authenticate3GPFile_1.3gp/ Authenticate3GPFile_2.3gp
	switch (pFilenameInfo->part)
	{
	case 0:
		// do nothing
		break;
	case 1:
		AVComposer_SetAuthenticity(pFilenameInfo->pAVComposer, "vvtk");
		break;
	case 2:
		AVComposer_SetAuthenticity(pFilenameInfo->pAVComposer, "VIVOTEK");
		break;
	}
	return S_OK;
}

int main()
{
	TCHAR szIP[MAX_IP_LENGTH] = {0};
	TCHAR szUserName[MAX_PASSWORD_LENGTH] = {0};
	TCHAR szPassword[MAX_USERNAME_LENGTH] = {0};
	TCHAR szServerType[MAX_SERVERTYPE_LENGTH] = {0};

	SCODE scRet = S_FAIL;
	HANDLE	hDataBroker = NULL;
	HANDLE	hConn = NULL;
	int nMaxChannel = 32;
		
	// STEP 1: Initial DataBroker
	scRet = DataBroker_Initial(&hDataBroker, nMaxChannel, DataBrokerStatusCallBack, DataBrokerAVCallBack, mctALLCODEC, 0, DATABROKER_VERSION);
	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Initial failed with error %X \n", scRet);
		return -1;
	}
	
	// STEP 2: Create connection
	scRet = DataBroker_CreateConnection(hDataBroker, &hConn);
	if (IS_FAIL(scRet))
	{
		printf("DataBroker_CreateConnection failed with error %X \n", scRet);
		return -1;
	}

	// STEP 3: Set option to connection
    // Open AVComposer handle pass to DataBrokerAVCallBack context
	TFilenameInfo filenameInfo = {};
#ifdef _UNICODE
	scRet = AVComposer_CreateHandleW(&filenameInfo.pAVComposer, 52428800, 2, &GetNewFilename, &filenameInfo);
#else
	scRet = AVComposer_CreateHandle(&filenameInfo.pAVComposer, 52428800, 2, &GetNewFilename, &filenameInfo);
#endif
	if (S_OK != scRet)
	{
		DataBroker_DeleteConnection(hDataBroker, &hConn);
		DataBroker_Release(&hDataBroker);
		return -4;
	}

	scRet = AVComposer_StartRecording(filenameInfo.pAVComposer, "Authenticate3GPFile_0.3gp", eRecOptVivoTrack);// eRecOptVivoTrack must be set, otherwise watermark doesn't take effect.
	if (IS_FAIL(scRet))
	{
		DataBroker_DeleteConnection(hDataBroker, &hConn);
		DataBroker_Release(&hDataBroker);
		return -4;
	}

	TDataBrokerConnectionOptions tOpt;
	memset(&tOpt, 0, sizeof(tOpt));
	tOpt.pzIPAddr = szIP;
	tOpt.pzUID = szUserName;
	tOpt.pzPWD = szPassword;
	tOpt.pzServerType = szServerType;
  
	CreateConnectionOption(&tOpt, filenameInfo.pAVComposer);               
	scRet = DataBroker_SetConnectionOptions(hConn, &tOpt);
	if (IS_FAIL(scRet))
	{
		printf("DataBroker_SetConnectionOptions failed with error %X \n", scRet);
		return -1;
	}

	// STEP 4: Connect to device, this function is an asynchronous function, 
	//		   the connection information should be get in StatusCallback function we set.
	scRet = DataBroker_Connect(hConn);
	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Connect failed with error %X \n", scRet);
		return -1;
	}

	// check the 3rd file is closed
	while (3 > filenameInfo.part)
	{
		// sleep 16 ms
		Sleep(16);
	}
		
	// STEP 5: Disconnect to device
	scRet = DataBroker_Disconnect(hConn);
	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Disconnect failed with error %X \n", scRet);
		return -1;
	}

    // STEP 6: Delete connection
    scRet = DataBroker_DeleteConnection(hDataBroker, &hConn);
	if (IS_FAIL(scRet))
	{
		printf("DataBroker_DeleteConnection failed with error %X \n", scRet);
		return -1;
	}

	// STEP 7: Release DataBroker
	scRet = DataBroker_Release(&hDataBroker);
	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Release failed with error %X \n", scRet);
		return -1;
	}
    
	// STEP 8: Close 3GP file and AVComposer handle
	scRet = AVComposer_StopRecording(filenameInfo.pAVComposer, NULL, FALSE);
	if (IS_FAIL(scRet))
	{
		printf("AVCOMPOSER_StopRecording failed with error %X \n", scRet);
		return -1;
	}
	scRet = AVComposer_ReleaseHandle(filenameInfo.pAVComposer);
	if (IS_FAIL(scRet))
	{
		printf("AVCOMPOSER_ReleaseHandle failed with error %X \n", scRet);
		return -1;
	}

	// STEP 9: Check authentication with Authenticate3GPFile_0.3gp using AVReader
	AVReader *pAVReader(NULL);
#ifdef _UNICODE
	scRet = AVReader_InitialW(&pAVReader, "Authenticate3GPFile_0.3gp", FALSE);
#else
	scRet = AVReader_Initial(&pAVReader, "Authenticate3GPFile_0.3gp", FALSE);
#endif
	if (S_OK != scRet)
	{
		printf("AVReader_Initial failed with error %X \n", scRet);
		return -1;
	}

	AVReader_SetAuthenticity(pAVReader, "VIVOTEK");// set user-defined key (e.g. "VIVOTEK") for checking watermark

	for (int i(1);; ++i)
	{
		TMediaDataPacketInfoV3 *ptMediaPacket(NULL);
		scRet = AVReader_OutputPacket(pAVReader, &ptMediaPacket);
		if (IS_FAIL(scRet))
		{
			AVReader_Release(pAVReader);
			printf("AVReader_OutputPacket failed with error %X \n", scRet);
			return -1;
		}
		else if (S_AVREADER_EOF == scRet)// terminated normally
		{
			AVReader_Release(pAVReader);
			break;
		}
		else if (S_AVREADER_NO_PROPER_AUTH == scRet)// expected result
		{
			printf("Authenticate3GPFile_0.3gp %ldst frame has no proper auth\n", i);
		}
		else
		{
			printf("Authenticate3GPFile_0.3gp %ldst frame is unexpected\n", i);
		}
	}

	// STEP 10: Check authentication with Authenticate3GPFile_1.3gp using AVReader
	pAVReader = NULL;
#ifdef _UNICODE
	scRet = AVReader_InitialW(&pAVReader, "Authenticate3GPFile_1.3gp", FALSE);
#else
	scRet = AVReader_Initial(&pAVReader, "Authenticate3GPFile_1.3gp", FALSE);
#endif
	if (S_OK != scRet)
	{
		printf("AVReader_Initial failed with error %X \n", scRet);
		return -1;
	}

	AVReader_SetAuthenticity(pAVReader, "VIVOTEK");// set key "VIVOTEK" for checking watermark

	for (int i(1);; ++i)
	{
		TMediaDataPacketInfoV3 *ptMediaPacket(NULL);
		scRet = AVReader_OutputPacket(pAVReader, &ptMediaPacket);
		if (IS_FAIL(scRet))
		{
			AVReader_Release(pAVReader);
			printf("AVReader_OutputPacket failed with error %X \n", scRet);
			return -1;
		}
		else if (S_AVREADER_EOF == scRet)// terminated normally
		{
			AVReader_Release(pAVReader);
			break;
		}
		else if (S_AVREADER_AUTH_NOT_MATCH == scRet)// expected result
		{
			printf("Authenticate3GPFile_1.3gp %ldst frame watermark not match\n", i);
		}
		else
		{
			printf("Authenticate3GPFile_1.3gp %ldst frame is unexpected\n", i);
		}
	}

	// STEP 11: Check authentication with Authenticate3GPFile_2.3gp using AVReader
	pAVReader = NULL;
#ifdef _UNICODE
	scRet = AVReader_InitialW(&pAVReader, "Authenticate3GPFile_2.3gp", FALSE);
#else
	scRet = AVReader_Initial(&pAVReader, "Authenticate3GPFile_2.3gp", FALSE);
#endif
	if (S_OK != scRet)
	{
		printf("AVReader_Initial failed with error %X \n", scRet);
		return -1;
	}

	AVReader_SetAuthenticity(pAVReader, "VIVOTEK");// set key "VIVOTEK" for checking watermark

	for (int i(1);; ++i)
	{
		TMediaDataPacketInfoV3 *ptMediaPacket(NULL);
		scRet = AVReader_OutputPacket(pAVReader, &ptMediaPacket);
		if (IS_FAIL(scRet))
		{
			AVReader_Release(pAVReader);
			printf("AVReader_OutputPacket failed with error %X \n", scRet);
			return -1;
		}
		else if (S_AVREADER_EOF == scRet)// terminated normally
		{
			AVReader_Release(pAVReader);
			break;
		}
		else if (S_OK == scRet)// expected result
		{
			printf("Authenticate3GPFile_2.3gp %ldst frame watermark is matched\n", i);
		}
		else
		{
			printf("Authenticate3GPFile_2.3gp %ldst frame is unexpected\n", i);
		}
	}

	return 0;
}
