#include "stdafx.h"
#include <memory.h>
#include "AvSyncLiveDigitalZoomWrapper.h"


// Function     : CAvSyncLiveDigitalZoomWrapper::CAvSyncLiveDigitalZoomWrapper
// Description  : Constructor
// Parameter    : 
//              | [in] hDlg             @Dialog window handle
//              | [in] hDisplay         @Display window handle
//              | [in] hBMP             @Bitmap handle
//              | [in] pfStatus         @Status callback function
//              | [in] pfDisplay        @Display callback function
//              | [in] dwStatusContext  @Status context
//              | [in] dwDisplayContext @Display context
// Return value : N/A
//
CAvSyncLiveDigitalZoomWrapper::CAvSyncLiveDigitalZoomWrapper(HWND hDlg, HWND hDisplay, HBITMAP hBMP, LPSTATUSCALLBACK pfStatus, LPDISPLAYCALLBACK pfDisplay, DWORD dwStatusContext, DWORD dwDisplayContext)
{
	m_hAvSync      = NULL;
	m_hLiveChannel = NULL;
	
	memset(&m_tParam, 0, sizeof(m_tParam));

	m_tParam.hDlg     = hDlg;
	m_tParam.hDisplay = hDisplay;
	m_tParam.hBMP     = hBMP;

	m_tParam.pfStatus  = pfStatus;
	m_tParam.pfDisplay = pfDisplay;
	
	m_tParam.dwStatusContext  = dwStatusContext;
	m_tParam.dwDisplayContext = dwDisplayContext;

	InitialModule();  // Initialize AvSynchronizer and create live channel
}

// Function     : CAvSyncLiveDigitalZoomWrapper::~CAvSyncLiveDigitalZoomWrapper
// Description  : Destructor
// Parameter    : N/A
// Return value : N/A
//
CAvSyncLiveDigitalZoomWrapper::~CAvSyncLiveDigitalZoomWrapper()
{	
	ReleaseModule();  // Delete live channel and release AvSynchronizer
}

// Function     : CAvSyncLiveDigitalZoomWrapper::InitialModule
// Description  : Initialize AvSynchronizer and create live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveDigitalZoomWrapper::InitialModule()
{	
	SCODE scRet;

	// STEP 1: Initialize AvSynchronizer
	scRet = AvSynchronizer_InitialEx(&m_hAvSync, m_tParam.pfStatus, m_tParam.pfDisplay, m_tParam.hDlg, AUDIOOUT_FOCUS_NORMAL, 0, AVSYNCHRONIZER_VERSION, 0, RGB(0, 0, 0));

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_InitialEx fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	if	(m_hAvSync != NULL)
	{
		// STEP 2: Create live channel
		scRet = CreateChannel(); 

		if	(IS_FAIL(scRet))
		{
			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveDigitalZoomWrapper::ReleaseModule
// Description  : Delete live channel and release AvSynchronizer
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveDigitalZoomWrapper::ReleaseModule()
{
	SCODE scRet;
	
	if	(m_hAvSync != NULL)
	{
		// STEP 1: Delete live channel
		if	(m_hLiveChannel != NULL)
		{
			scRet = DeleteChannel();

			if	(IS_FAIL(scRet))
			{
				return S_FAIL;
			}
		}

		// STEP 2: Release AvSynchronizer
		scRet = AvSynchronizer_Release(&m_hAvSync);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_Release fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveDigitalZoomWrapper::CreateChannel
// Description  : Create live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveDigitalZoomWrapper::CreateChannel()
{	
	// STEP 1: Set channel options
	TCHANNELOPTION tCHOption;

	memset(&tCHOption, 0, sizeof(tCHOption));

	tCHOption.dwFlags = CH_STATUSCB | CH_DISPLAYCB | CH_STATUSCONTEXT | CH_DISPLAYCONTEXT | CH_ALLBORDER | CH_BITMAP;

	tCHOption.pfStatus  = m_tParam.pfStatus; 
	tCHOption.pfDisplay = m_tParam.pfDisplay;

	tCHOption.dwStatusContext = m_tParam.dwStatusContext;
	tCHOption.dwDisplayContext = m_tParam.dwDisplayContext;
	
	tCHOption.dwTopBorderSize    = 20;
	tCHOption.dwLeftBorderSize   = 0;
	tCHOption.dwBottomBorderSize = 0;
	tCHOption.dwRightBorderSize  = 0;

	tCHOption.hBMP     = m_tParam.hBMP;
	tCHOption.hDisplay = m_tParam.hDisplay;
	
	SCODE scRet;

	// STEP 2: Create live channel
	scRet = AvSynchronizer_CreateChannel(m_hAvSync, &m_hLiveChannel, tCHOption);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_CreateChannel fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}
	
	return S_OK;
}

// Function     : CAvSyncLiveDigitalZoomWrapper::DeleteChannel
// Description  : Delete live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveDigitalZoomWrapper::DeleteChannel()
{
	SCODE scRet;

	// Delete live channel 
	scRet = AvSynchronizer_DeleteChannel(m_hAvSync, &m_hLiveChannel);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_DeleteChannel fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	return S_OK;
}

// Function     : CAvSyncLiveDigitalZoomWrapper::Start
// Description  : Start live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveDigitalZoomWrapper::Start()
{	
	SCODE scRet;

	// Start live channel
	if	(m_hLiveChannel != NULL)
	{
		scRet = AvSynchronizer_StartChannel(m_hLiveChannel, DRAW_CONNECTING);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_StartChannel fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveDigitalZoomWrapper::Stop
// Description  : Stop live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveDigitalZoomWrapper::Stop()
{
	SCODE scRet;
	
	// Stop live channel
	if	(m_hLiveChannel != NULL)
	{
		scRet = AvSynchronizer_StopChannel(m_hLiveChannel);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_StopChannel fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveDigitalZoomWrapper::UpdateChannelMediaType
// Description  : Update channel media type
// Parameter    :
//              | [in] dwMediaType @Media type
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveDigitalZoomWrapper::UpdateChannelMediaType(DWORD dwMediaType)
{
	SCODE scRet;

	if	(m_hLiveChannel != NULL)
	{	
		TUPDATECHANNELOPTION tUpdataCHOption;

		tUpdataCHOption.dwFlags    = UPCH_MEDIATYPE;
		tUpdataCHOption.tMediaType = EMEDIATYPE(dwMediaType);
	
		scRet = AvSynchronizer_UpdateChannelSettings(m_hLiveChannel, tUpdataCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdateChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveDigitalZoomWrapper::UpdateChannelShowBMP
// Description  : Update channel settings to show bitmap
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveDigitalZoomWrapper::UpdateChannelShowBMP()
{
	SCODE scRet;

	if	(m_hLiveChannel != NULL)
	{	
		TUPDATECHANNELOPTION tUpdataCHOption;

		tUpdataCHOption.dwFlags = UPCH_SHOW_BMP;
	
		scRet = AvSynchronizer_UpdateChannelSettings(m_hLiveChannel, tUpdataCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdateChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveDigitalZoomWrapper::InputEncodedMediaFrame
// Description  : Input the encoded media frame into channel
// Parameter    :
//              | [in] pMediaDataPacket @Input media data packet info
// Return value :
//              | S_OK                        @Success
//              | S_FAIL                      @FAIL
//              | AVSYNCHRONIZER_E_QUEUE_FULL @Queue full
//
SCODE CAvSyncLiveDigitalZoomWrapper::InputEncodedMediaFrame(TMediaDataPacketInfo *pMediaDataPacket)
{
	SCODE scRet;

	if	(m_hLiveChannel != NULL)
	{		
		scRet = AvSynchronizer_InputEncodedMediaFrame(m_hLiveChannel, pMediaDataPacket);

		if (scRet != S_OK)
		{
			// The return value "AVSYNCHRONIZER_E_QUEUE_FULL" is used to notify the caller that the queue is full.
			// When used in DataBrokerAVCallback function, if the callback function recieves "AVSYNCHRONIZER_E_QUEUE_FULL",
			// it returns "DATABROKER_S_FRAME_NOT_HANDLED" back to DataBroker. Then DataBroker stop putting media data into
			// the queue until it recieves "S_OK."
			//
			if	(scRet == AVSYNCHRONIZER_E_QUEUE_FULL)
			{
				sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer queue full!\n", scRet);

				OutputDebugString(m_szErrMsgBuf);

				return scRet;
			}
			else
			{
				sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_InputEncodedMediaFrame fail! (ERROR: %X)\n", scRet);

				OutputDebugString(m_szErrMsgBuf);

				return S_FAIL;
			}
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveDigitalZoomWrapper::SetDigitalZoomOptions
// Description  : Set the channel option to enable digital zoom capibility
// Parameter    :
//              | [in] hTarget @Handle of target window holds full view of video
//              | [in] hZoomas @Handle of selected zoom area window 
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveDigitalZoomWrapper::SetDigitalZoomOptions(HWND hTarget, HWND hZoomas)
{
	// STEP 1: Set digital zoom options
	TDIGITALZOOMPARAM tDZParam;

	memset(&tDZParam, 0, sizeof(tDZParam));

	DWORD dwCanvasBorderSize   = 4;
	DWORD dwZoomAreaBorderSize = 4; 

	// Set the "bCanvasWin" to TRUE if user wants to use the taget window handle as a reference.
	// Otherwise, set the "bCanvasWin" to FALSE if user wants to use the taget window coordinate as a reference.
	// For example, if the "bCanvasWin" is set to FALSE, the "tDZParam.unCanvas.rectTarget" is now valid. User
	// should pass the coordinate (RECT) to the option. In this sample code,  we set bCanvasWin to TRUE.
	//
	tDZParam.bCanvasWin       = TRUE;
	tDZParam.unCanvas.hTarget = hTarget;
	
	tDZParam.rectCanvasBorder.top    = dwCanvasBorderSize;
	tDZParam.rectCanvasBorder.left   = dwCanvasBorderSize;
	tDZParam.rectCanvasBorder.bottom = dwCanvasBorderSize;
	tDZParam.rectCanvasBorder.right  = dwCanvasBorderSize;
	
	// Set the "bZoomWin" to TRUE if user wants to use the area of interest (ZoomArea) window handle as a reference.
	// Otherwise, set the "bZoomWin" to FALSE if user wants to use the ZoomArea window coordinate as a reference.
	// For example, if the "bZoomWin" is set to FALSE, the "tDZParam.unZoom.rectZoomArea" is now valid. User
	// should pass the coordinate (RECT) to the option. In this sample code,  we set bZoomWin to TRUE.
	//
	tDZParam.bZoomWin         = TRUE;	
	tDZParam.unZoom.hZoomArea = hZoomas;

	tDZParam.rectZoomBorder.top    = dwZoomAreaBorderSize;
	tDZParam.rectZoomBorder.left   = dwZoomAreaBorderSize;
	tDZParam.rectZoomBorder.bottom = dwZoomAreaBorderSize;
	tDZParam.rectZoomBorder.right  = dwZoomAreaBorderSize;

	tDZParam.bZoomEnabled = TRUE;
	tDZParam.bDisplayed   = TRUE;

	SCODE scRet;

	// STEP 2: Set channel options (perform digital zoom)
	DWORD dwDZFlags = DZ_CANVAS | DZ_CANVAS_DISPLAY | DZ_ZOOM_AREA | DZ_ENABLE_ZOOM | DZ_CANVAS_BORDER | DZ_ZOOM_AREA_BORDER;

	scRet = AvSynchronizer_SetChannelOption(m_hLiveChannel, SETCH_DIGITAL_ZOOM, (DWORD)(&tDZParam), dwDZFlags);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_SetChannelOption fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	return S_OK;
}
