#include "DataBroker.h"
#include "OSDefine.h"


// This callback function is used to receive media packet, the calling convention must be stdcall
SCODE __stdcall DataBrokerAVCallBack(DWORD dwContext, TMediaDataPacketInfo *ptMediaDataPacket)
{	
	return S_OK;
}

// This callback function is used to receive status info, the calling convention must be stdcall
SCODE __stdcall DataBrokerStatusCallBack(DWORD dwContext, TDataBrokerStatusType tStatusType, PVOID pvParam1, PVOID pvParam2)
{	
	switch (tStatusType)
	{
		case eOnConnectionInfo:
			printf("Get eOnConnectionInfo, start to receive packet\n");
			break;
		case eOnStopped:
			printf("Get eOnStopped, stop to receive packet\n");
			break;
		default:
			break;
	}
	
	return S_OK;
}

int main(int argc, char *argv[])
{
	SCODE scRet;
	HANDLE hDataBroker = NULL;
	HANDLE hConnection = NULL;
	int iMaxChannel = 32;
	TDataBrokerConnectionOptions tDBConOption = {0};
	char szUrl[] = "/Live/Normal?camera=C_1&streamindex=1"; // Live streaming sample URL
	const DWORD dwRTSPPort = 3454; // Darwin server default RTSP port is 3454
	
	scRet = DataBroker_Initial(&hDataBroker, iMaxChannel, DataBrokerStatusCallBack, DataBrokerAVCallBack, (DWORD)mctALLCODEC, 0, DATABROKER_VERSION);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Initial failed with error %X \n", scRet);
		return -1;
	}
	
	scRet = DataBroker_CreateConnection(hDataBroker, &hConnection);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_CreateConnection failed with error %X \n", scRet);
		return -1;
	}
	
	tDBConOption.wHttpPort = 80;
	tDBConOption.dwProtocolType = eptTCP;
	tDBConOption.dwMediaType = emtVideo | emtAudio;
	tDBConOption.pzServerType = "Darwin";
	tDBConOption.pzIPAddr = "127.0.0.1";	
	tDBConOption.pzUID = "admin";
	tDBConOption.pzPWD = "admin";	
	tDBConOption.dwFlags = eConOptProtocolAndMediaType | eConOptHttpPort;
	
	scRet = DataBroker_SetConnectionOptions(hConnection, &tDBConOption);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_SetConnectionOptions failed with error %X \n", scRet);
		return -1;
	}
	
	scRet = DataBroker_SetConnectionUrlsExtra(hConnection, szUrl, NULL, NULL, NULL, NULL, NULL);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_SetConnectionUrlsExtra failed with error %X \n", scRet);
		return -1;
	}
	
	scRet = DataBroker_SetConnectionExtraOption(hConnection, eOptRtspCtrlPort, dwRTSPPort, 0);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_SetConnectionExtraOption failed with error %X \n", scRet);
		return -1;
	}
	
	scRet = DataBroker_Connect(hConnection);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Connect failed with error %X \n", scRet);
		return -1;
	}
	
	// Press any key to stop the receiving.
	while (!_kbhit())
	{
		Sleep(16);
	}
		
	scRet = DataBroker_Disconnect(hConnection);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Disconnect failed with error %X \n", scRet);
		return -1;
	}
	
	scRet = DataBroker_DeleteConnection(hDataBroker, &hConnection);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_DeleteConnection failed with error %X \n", scRet);
		return -1;
	}
	
	scRet = DataBroker_Release(&hDataBroker);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Release failed with error %X \n", scRet);
		return -1;
	}
	
	hConnection = NULL;
	hDataBroker = NULL;
	
	return 0;
}